/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.components.xsql;

import org.servicemix.components.util.ComponentSupport;
import org.servicemix.components.util.TransformComponentSupport;
import org.servicemix.MessageExchangeListener;
import org.servicemix.jbi.NoInMessageAvailableException;
import org.servicemix.jbi.jaxp.SourceTransformer;
import org.w3c.dom.Document;
import org.springframework.core.io.Resource;
import org.xml.sax.SAXException;
import oracle.xml.xsql.XSQLConnectionManagerFactory;
import oracle.xml.xsql.XSQLRequest;
import oracle.xml.parser.v2.XMLDocument;
import oracle.xml.parser.v2.DOMParser;
import oracle.xml.parser.v2.DocumentBuilder;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.JBIException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.TransformerException;
import java.net.URL;
import java.io.IOException;

/**
 * A JBI component for interacting with
 * <a href="http://www.oracle.com/technology/tech/xml/xdkhome.html">Oracle XSQL</a>
 *
 * @version $Revision: 1.2 $
 */
public class XSQLComponent extends TransformComponentSupport implements MessageExchangeListener {
    private XSQLConnectionManagerFactory connectionManagerFactory;
    private URL baseURL;
    private XMLDocument xsql;
    private Resource xsqlResource;
    private SourceTransformer transformer = new SourceTransformer();

    protected void init() throws JBIException {
        super.init();
        if (xsql == null) {
            if (xsqlResource != null) {
                try {
                    DOMParser parser = new DOMParser();
                    parser.parse(xsqlResource.getInputStream());
                    xsql = parser.getDocument();
                }
                catch (Exception e) {
                    throw new JBIException("Failed to parse: " + xsqlResource +". Reason: " + e, e);
                }
            }
        }
    }

    // Properties
    //-------------------------------------------------------------------------
    public XSQLConnectionManagerFactory getConnectionManagerFactory() {
        return connectionManagerFactory;
    }

    public void setConnectionManagerFactory(XSQLConnectionManagerFactory connectionManagerFactory) {
        this.connectionManagerFactory = connectionManagerFactory;
    }

    public URL getBaseURL() {
        return baseURL;
    }

    public void setBaseURL(URL baseURL) {
        this.baseURL = baseURL;
    }

    public XMLDocument getXsql() {
        return xsql;
    }

    public void setXsql(XMLDocument xsql) {
        this.xsql = xsql;
    }

    public Resource getXsqlResource() {
        return xsqlResource;
    }

    public void setXsqlResource(Resource xsqlResource) {
        this.xsqlResource = xsqlResource;
    }

    public SourceTransformer getTransformer() {
        return transformer;
    }

    public void setTransformer(SourceTransformer transformer) {
        this.transformer = transformer;
    }

    // Implementation methods
    //-------------------------------------------------------------------------
    protected void transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws MessagingException {
        XMLDocument document = createXSQLDocument(exchange, in);
        XSQLRequest request = createXSQLRequest(exchange, in, document);
        Document response = request.processToXML();
        out.setContent(new DOMSource(response));
    }

    protected XSQLRequest createXSQLRequest(MessageExchange exchange, NormalizedMessage in, XMLDocument document) {
        return new XSQLRequest(connectionManagerFactory, document, baseURL);
    }

    protected XMLDocument createXSQLDocument(MessageExchange exchange, NormalizedMessage in) throws MessagingException {
        if (xsql != null) {
            return xsql;
        }
        try {
            DocumentBuilder builder = new DocumentBuilder();
            SAXResult result = new SAXResult(builder);
            transformer.toResult(in.getContent(), result);
            return builder.getDocument();
        }
        catch (TransformerException e) {
            throw new MessagingException("Failed to convert inbound message content to an XMLDocument. Reason: " + e, e);
        }
    }
}
