package org.servicemix.jbi.container;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.servicemix.components.util.ComponentAdaptor;
import org.servicemix.components.util.ComponentAdaptorMEListener;
import org.servicemix.components.util.ComponentSupport;
import org.servicemix.components.util.PojoSupport;
import org.servicemix.MessageExchangeListener;
import org.servicemix.jbi.framework.LocalComponentConnector;
import org.servicemix.components.util.ComponentAdaptor;
import org.servicemix.components.util.ComponentAdaptorMEListener;
import org.servicemix.components.util.ComponentSupport;
import org.servicemix.components.util.PojoSupport;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.activemq.util.IdGenerator;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.xml.namespace.QName;
import java.util.Iterator;
import java.util.List;


/**
 * An enhanced JBI container which adds some Spring helper methods for
 * easier configuration through spring's XML configuration file.
 *
 * @version $Revision: 1.16 $
 */
public class SpringJBIContainer extends JBIContainer implements InitializingBean, BeanFactoryAware {
    private static final transient Log log = LogFactory.getLog(SpringJBIContainer.class);

    private String[] componentNames;
    private List activationSpecs;
    private BeanFactory beanFactory;
    private String[] deployArchives;

    public void afterPropertiesSet() throws Exception {
        init();

        // lets iterate through all the component names and register them
        if (componentNames != null) {
            for (int i = 0; i < componentNames.length; i++) {
                String componentName = componentNames[i];
                activateComponent(new ActivationSpec(componentName, lookupBean(componentName)));
            }
        }

        if (activationSpecs != null) {
            for (Iterator iter = activationSpecs.iterator(); iter.hasNext();) {
                ActivationSpec activationSpec = (ActivationSpec) iter.next();
                activateComponent(activationSpec);
            }
        }

        if (deployArchives != null) {
            for (int i = 0; i < deployArchives.length; i++) {
                String archive = deployArchives[i];
                installArchive(archive);
            }
        }

        start();
    }

    public void stop() throws JBIException {
        if (beanFactory instanceof DisposableBean) {
            DisposableBean disposable = (DisposableBean) beanFactory;
            try {
                disposable.destroy();
            }
            catch (Exception e) {
                throw new JBIException("Failed to dispose of the Spring BeanFactory due to: " + e, e);
            }
        }
        super.stop();
    }

    /**
     * Returns the compoment or POJO registered with the given component ID.
     *
     * @param id
     * @return the Component
     *
     */
    public Object getBean(String id) {
        Object bean = getComponent(id);
        if (bean instanceof ComponentAdaptor) {
            ComponentAdaptor adaptor = (ComponentAdaptor) bean;
            return adaptor.getLifeCycle();
        }
        return bean;
    }


    // Properties
    //-------------------------------------------------------------------------
    public BeanFactory getBeanFactory() {
        return beanFactory;
    }

    public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
        this.beanFactory = beanFactory;
    }

    public String[] getComponentNames() {
        return componentNames;
    }

    public void setComponentNames(String[] componentNames) {
        this.componentNames = componentNames;
    }

    public ServiceUnitManager getServiceManager() {
        return serviceManager;
    }

    public void setServiceManager(ServiceUnitManager serviceManager) {
        this.serviceManager = serviceManager;
    }

    public void setActivationSpecs(List activationSpecs) throws JBIException {
        this.activationSpecs = activationSpecs;
    }

    public String[] getDeployArchives() {
        return deployArchives;
    }

    public void setDeployArchives(String[] deployArchives) {
        this.deployArchives = deployArchives;
    }

    // Implementation methods
    //-------------------------------------------------------------------------
    protected Object lookupBean(String componentName) {
        Object bean = beanFactory.getBean(componentName);
        if (bean == null) {
            throw new IllegalArgumentException("Component name: " + componentName
                    + " is not found in the Spring BeanFactory");
        }
        return bean;
    }

}
