package org.servicemix.examples;

import junit.framework.TestCase;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.AbstractXmlApplicationContext;
import org.servicemix.jbi.container.SpringJBIContainer;

/**
 * A base class for spring related test cases
 *
 * @version $Revision: 1.7 $
 */
public abstract class SpringTestSupport extends TestCase {
    protected SpringJBIContainer jbi;
    protected AbstractXmlApplicationContext context;
    protected int messageCount = 10;

    protected void setUp() throws Exception {

        context = createBeanFactory();
        context.setXmlValidating(false);

        // lets force the JBI container to be constructed first
        jbi = (SpringJBIContainer) context.getBean("jbi");
        assertNotNull("JBI Container not found in spring!", jbi);

    }

    protected abstract AbstractXmlApplicationContext createBeanFactory();

    public void testSendingAndReceivingMessagesUsingSpring() throws Exception {
        Sender sender = getSender();
        Receiver receiver = getReceiver();

        sender.sendMesssages(messageCount);

        MessageList messageList = getReceivedMessageList();
        messageList.assertMessagesReceived(messageCount);
    }

    protected Sender getSender() {
        return (Sender) getBean("sender");
    }

    protected Receiver getReceiver() {
        return (Receiver) getBean("receiver");
    }

    protected MessageList getReceivedMessageList() {
        return getReceiver().getMessageList();
    }

    protected Object getBean(String name) {
        Object value = jbi.getBean(name);
        if (value == null) {
            value = context.getBean(name);
        }
        assertNotNull(name + " not found in JBI container!", value);
        return value;
    }
}
